#define IDD_FOLDERS 571
#define IDD_FOLDERS_2 671

#define IDS_FOLDERS_SET_WORK_PATH_TITLE  877

#define IDC_FOLDERS_STATIC_WORKING_FOLDER 1001

#define IDC_FOLDERS_WORK_RADIO_SYSTEM    1011
#define IDC_FOLDERS_WORK_RADIO_CURRENT   1012
#define IDC_FOLDERS_WORK_RADIO_SPECIFIED 1013
#define IDC_FOLDERS_WORK_EDIT_PATH       1014
#define IDC_FOLDERS_WORK_BUTTON_PATH     1015
#define IDC_FOLDERS_WORK_CHECK_FOR_REMOVABLE 1017
