#define IDD_DIALOG_PROGRESS             500
#define IDD_DIALOG_PROGRESS_2           600

#define IDC_BUTTON_PAUSE                50
#define IDC_BUTTON_PROGRESS_PRIORITY    51

#define IDS_PROGRESS_PAUSED             700
#define IDS_PROGRESS_FOREGROUND         701
#define IDS_PROGRESS_CONTINUE           702
#define IDS_PROGRESS_ASK_CANCEL         703
#define IDS_CLOSE                       704
#define IDS_MESSAGES_DIALOG_MESSAGE_COLUMN 503

#define IDC_PROGRESS1                   1000
#define IDC_PROGRESS_ELAPSED            1002
#define IDC_PROGRESS_ELAPSED_VALUE      1003
#define IDC_PROGRESS_REMAINING          1004
#define IDC_PROGRESS_REMAINING_VALUE    1005
#define IDC_PROGRESS_SPEED              1006
#define IDC_PROGRESS_SPEED_VALUE        1007
#define IDC_PROGRESS_TOTAL              1008
#define IDC_PROGRESS_TOTAL_VALUE        1009
#define IDC_PROGRESS_FILE_NAME          1010

#define IDC_PROGRESS_FILES              1012
#define IDC_PROGRESS_FILES_VALUE        1013
#define IDC_PROGRESS_RATIO              1014
#define IDC_PROGRESS_RATIO_VALUE        1015
#define IDC_PROGRESS_PACKED             1016
#define IDC_PROGRESS_PACKED_VALUE       1017
#define IDC_PROGRESS_UNPACKED           1018
#define IDC_PROGRESS_UNPACKED_VALUE     1019

#define IDC_PROGRESS_ERRORS         1030
#define IDC_PROGRESS_ERRORS_VALUE   1031
#define IDC_PROGRESS_LIST           1032

#define MY_PROGRESS_VALUE_UNITS  44
#define MY_PROGRESS_LABEL_UNITS_MIN  60
#define MY_PROGRESS_LABEL_UNITS_START 90
#define MY_PROGRESS_PAD_UNITS  4
