#include "LangPageRes.h"
#include "../../GuiCommon.rc"

#define xc 148
#define yc 100

IDD_LANG  MY_PAGE
CAPTION "Language"
{
  LTEXT     "Language:", IDC_LANG_STATIC_LANG, m, m, xc, 8
  COMBOBOX  IDC_LANG_COMBO_LANG, m, 20, xc, yc - 20, MY_COMBO_SORTED
}


#ifdef UNDER_CE

#undef m
#undef xc

#define m 4
#define xc (SMALL_PAGE_SIZE_X + 8)

IDD_LANG_2  MY_PAGE
CAPTION "Language"
{
  LTEXT     "Language:", IDC_LANG_STATIC_LANG, m, m, xc, 8
  COMBOBOX  IDC_LANG_COMBO_LANG, m, 20, xc, yc - 20, MY_COMBO_SORTED
}

#endif


STRINGTABLE
BEGIN
  IDS_LANG_ENGLISH  "English"
  IDS_LANG_NATIVE   "English"
END
