#define IDD_MENU 570
#define IDD_MENU_2 670

#define IDC_SYSTEM_INTEGRATE_TO_CONTEXT_MENU 1010
#define IDC_SYSTEM_CASCADED_MENU        1011
#define IDC_SYSTEM_STATIC_CONTEXT_MENU_ITEMS 1020
#define IDC_SYSTEM_OPTIONS_LIST         1022
