#define IDS_CONTEXT_EXTRACT             142
#define IDS_CONTEXT_EXTRACT_HELP        143
#define IDS_CONTEXT_COMPRESS            144
#define IDS_CONTEXT_COMPRESS_HELP       145
#define IDS_CONTEXT_OPEN                146
#define IDS_CONTEXT_OPEN_HELP           147
#define IDS_CONTEXT_TEST                148
#define IDS_CONTEXT_TEST_HELP           149
#define IDS_CONTEXT_CAPTION_HELP        150
#define IDS_CONTEXT_POPUP_CAPTION       151
                                   
#define IDS_CONTEXT_EXTRACT_HERE           152
#define IDS_CONTEXT_EXTRACT_HERE_HELP      153

#define IDS_CONTEXT_EXTRACT_TO           154
#define IDS_CONTEXT_EXTRACT_TO_HELP      155

#define IDS_CONTEXT_COMPRESS_TO          156
#define IDS_CONTEXT_COMPRESS_TO_HELP     157

#define IDS_CONTEXT_COMPRESS_EMAIL       158
#define IDS_CONTEXT_COMPRESS_EMAIL_HELP  159

#define IDS_CONTEXT_COMPRESS_TO_EMAIL       160
#define IDS_CONTEXT_COMPRESS_TO_EMAIL_HELP  161

#define IDS_CONTEXT_FOLDER  170
#define IDS_CONTEXT_ARCHIVE 171
