#define IDD_DIALOG_EXTRACT              552
#define IDD_DIALOG_EXTRACT_2            652

#define IDC_STATIC_EXTRACT_EXTRACT_TO   1020
#define IDC_EXTRACT_COMBO_PATH          1021
#define IDC_EXTRACT_BUTTON_SET_PATH     1022

#define IDC_EXTRACT_COMBO_PATH_MODE 1030
#define IDC_EXTRACT_COMBO_OVERWRITE_MODE 1031

#define IDC_EXTRACT_PATH_MODE           1040
#define IDC_EXTRACT_OVERWRITE_MODE      1050

#define IDC_EXTRACT_FILES               1060

#define IDC_EXTRACT_PASSWORD            1100
#define IDC_EXTRACT_EDIT_PASSWORD       1101
#define IDC_EXTRACT_CHECK_SHOW_PASSWORD 1102

