#define IDD_DIALOG_BENCHMARK            550
#define IDD_DIALOG_BENCHMARK_2          650
#define IDC_BUTTON_STOP                 1001
#define IDC_BUTTON_RESTART              1002
#define IDC_BENCHMARK_DICTIONARY        1010
#define IDC_BENCHMARK_COMBO_DICTIONARY  1011
#define IDC_BENCHMARK_MEMORY            1012
#define IDC_BENCHMARK_MEMORY_VALUE      1013
#define IDC_BENCHMARK_NUM_THREADS       1014
#define IDC_BENCHMARK_COMBO_NUM_THREADS 1015
#define IDC_BENCHMARK_HARDWARE_THREADS  1016

#define IDC_BENCHMARK_SPEED_LABEL       1020
#define IDC_BENCHMARK_RATING_LABEL      1021
#define IDC_BENCHMARK_COMPRESSING       1022
#define IDC_BENCHMARK_DECOMPRESSING     1023
#define IDC_BENCHMARK_CURRENT           1024
#define IDC_BENCHMARK_RESULTING         1025
#define IDC_BENCHMARK_CURRENT2          1026
#define IDC_BENCHMARK_RESULTING2        1027
#define IDC_BENCHMARK_USAGE_LABEL       1028
#define IDC_BENCHMARK_RPU_LABEL         1029

#define IDC_BENCHMARK_COMPRESSING_SPEED 1030
#define IDC_BENCHMARK_COMPRESSING_SPEED2 1031
#define IDC_BENCHMARK_COMPRESSING_RATING 1032
#define IDC_BENCHMARK_COMPRESSING_RATING2 1033
#define IDC_BENCHMARK_COMPRESSING_USAGE 1034
#define IDC_BENCHMARK_COMPRESSING_USAGE2 1035
#define IDC_BENCHMARK_COMPRESSING_RPU 1036
#define IDC_BENCHMARK_COMPRESSING_RPU2 1037


#define IDC_BENCHMARK_DECOMPRESSING_SPEED 1040
#define IDC_BENCHMARK_DECOMPRESSING_SPEED2 1041
#define IDC_BENCHMARK_DECOMPRESSING_RATING 1042
#define IDC_BENCHMARK_DECOMPRESSING_RATING2 1043
#define IDC_BENCHMARK_DECOMPRESSING_USAGE 1044
#define IDC_BENCHMARK_DECOMPRESSING_USAGE2 1045
#define IDC_BENCHMARK_DECOMPRESSING_RPU 1046
#define IDC_BENCHMARK_DECOMPRESSING_RPU2 1047


#define IDC_BENCHMARK_TOTAL_RATING      1050

#define IDC_BENCHMARK_TOTAL_RATING_VALUE 1051
#define IDC_BENCHMARK_TOTAL_RPU_VALUE 1052
#define IDC_BENCHMARK_TOTAL_USAGE_VALUE 1053

#define IDC_BENCHMARK_ELAPSED           1060
#define IDC_BENCHMARK_ELAPSED_VALUE     1061
#define IDC_BENCHMARK_SIZE              1062
#define IDC_BENCHMARK_SIZE_VALUE        1063
#define IDC_BENCHMARK_PASSES            1066
#define IDC_BENCHMARK_PASSES_VALUE      1067
