EXPORTS
  CreateObject PRIVATE
  GetHandlerProperty PRIVATE
  GetNumberOfFormats PRIVATE
  GetHandlerProperty2 PRIVATE
  CreateObject PRIVATE
  GetNumberOfMethods PRIVATE
  GetMethodProperty PRIVATE
  SetLargePageMode PRIVATE
