#include "resourceGui.h"

#define IDR_MENUBAR1                    103
#define IDM_MENU                        103
#define IDR_ACCELERATOR1                209
#define IDM_FILE_OPEN                   210
#define IDM_FILE_OPEN_INSIDE            211
#define IDM_FILE_OPEN_OUTSIDE           212
#define IDM_FILE_VIEW                   220
#define IDM_FILE_EDIT                   221
#define IDM_RENAME                      230
#define IDM_COPY_TO                     231
#define IDM_MOVE_TO                     232
#define IDM_DELETE                      233
#define IDM_FILE_SPLIT                  238
#define IDM_FILE_COMBINE                239
#define IDM_FILE_PROPERTIES             240
#define IDM_FILE_COMMENT                241
#define IDM_FILE_CRC                    242
#define IDM_FILE_DIFF                   243
#define IDM_CREATE_FOLDER               250
#define IDM_CREATE_FILE                 251
#define IDM_EDIT_CUT                    320
#define IDM_EDIT_COPY                   321
#define IDM_EDIT_PASTE                  322
#define IDM_SELECT_ALL                  330
#define IDM_DESELECT_ALL                331
#define IDM_INVERT_SELECTION            332
#define IDM_SELECT                      333
#define IDM_DESELECT                    334
#define IDM_SELECT_BY_TYPE              335
#define IDM_DESELECT_BY_TYPE            336
#define IDM_VIEW_LARGE_ICONS            410
#define IDM_VIEW_SMALL_ICONS            411
#define IDM_VIEW_LIST                   412
#define IDM_VIEW_DETAILS                413
#define IDM_VIEW_ARANGE_BY_NAME         420
#define IDM_VIEW_ARANGE_BY_TYPE         421
#define IDM_VIEW_ARANGE_BY_DATE         422
#define IDM_VIEW_ARANGE_BY_SIZE         423
#define IDM_VIEW_ARANGE_NO_SORT         424
#define IDM_OPEN_ROOT_FOLDER            430
#define IDM_OPEN_PARENT_FOLDER          431
#define IDM_FOLDERS_HISTORY             432
#define IDM_VIEW_REFRESH                440
#define IDM_VIEW_FLAT_VIEW              449
#define IDM_VIEW_TWO_PANELS             450
#define IDM_VIEW_TOOLBARS               451
#define IDM_VIEW_STANDARD_TOOLBAR       460
#define IDM_VIEW_ARCHIVE_TOOLBAR        461
#define IDM_VIEW_TOOLBARS_LARGE_BUTTONS 462
#define IDM_VIEW_TOOLBARS_SHOW_BUTTONS_TEXT 463
#define IDM_OPTIONS                     510
#define IDM_BENCHMARK                   511
#define IDM_HELP_CONTENTS               610
#define IDM_ABOUT                       620
#define IDS_BOOKMARK                    720
#define IDB_ADD                         2002
#define IDB_EXTRACT                     2003
#define IDB_TEST                        2004
#define IDB_COPY                        2010
#define IDB_MOVE                        2011
#define IDB_DELETE                      2012
#define IDB_INFO                        2013
#define IDB_ADD2                        2082
#define IDB_EXTRACT2                    2083
#define IDB_TEST2                       2084
#define IDB_COPY2                       2090
#define IDB_MOVE2                       2091
#define IDB_DELETE2                     2092
#define IDB_INFO2                       2093
#define IDS_APP_TITLE                   2200
#define IDS_COPY                        2201
#define IDS_MOVE                        2202
#define IDS_COPY_TO                     2203
#define IDS_MOVE_TO                     2204
#define IDS_COPYING                     2205
#define IDS_MOVING                      2206
#define IDS_CANNOT_COPY                 2207
#define IDS_OPERATION_IS_NOT_SUPPORTED  2208

#define IDS_CONFIRM_FILE_DELETE         2210
#define IDS_CONFIRM_FOLDER_DELETE       2211
#define IDS_CONFIRM_ITEMS_DELETE        2212
#define IDS_WANT_TO_DELETE_FILE         2213
#define IDS_WANT_TO_DELETE_FOLDER       2214
#define IDS_WANT_TO_DELETE_ITEMS        2215
#define IDS_DELETING                    2216
#define IDS_ERROR_DELETING              2217
#define IDS_ERROR_LONG_PATH_TO_RECYCLE  2218

#define IDS_RENAMING                    2220
#define IDS_ERROR_RENAMING              2221
#define IDS_CONFIRM_FILE_COPY           2222
#define IDS_WANT_TO_COPY_FILES          2223


#define IDS_CREATE_FOLDER               2230
#define IDS_CREATE_FOLDER_NAME          2231
#define IDS_CREATE_FOLDER_DEFAULT_NAME  2232
#define IDS_CREATE_FOLDER_ERROR         2233
#define IDS_CREATE_FILE                 2240
#define IDS_CREATE_FILE_NAME            2241
#define IDS_CREATE_FOLDER_DEFAULT_FILE_NAME 2242
#define IDS_CREATE_FILE_DEFAULT_NAME    2242
#define IDS_CREATE_FILE_ERROR           2243
#define IDS_SELECT                      2250
#define IDS_DESELECT                    2251
#define IDS_SELECT_MASK                 2252
#define IDS_FOLDERS_HISTORY             2260
#define IDS_N_SELECTED_ITEMS            2270

#define IDS_TOO_MANY_ITEMS              2279
#define IDS_WANT_UPDATE_MODIFIED_FILE   2280
#define IDS_CANNOT_UPDATE_FILE          2281
#define IDS_CANNOT_START_EDITOR         2282
#define IDS_OPENNING                    2283
#define IDS_VIRUS                       2284
#define IDS_COMPUTER                    2300
#define IDS_NETWORK                     2301
#define IDS_DOCUMENTS                   2302
#define IDS_ADD                         2400
#define IDS_EXTRACT                     2401
#define IDS_TEST                        2402
#define IDS_BUTTON_COPY                 2420
#define IDS_BUTTON_MOVE                 2421
#define IDS_BUTTON_DELETE               2422
#define IDS_BUTTON_INFO                 2423
#define IDS_PROP_TOTAL_SIZE             3100
#define IDS_PROP_FREE_SPACE             3101
#define IDS_PROP_CLUSTER_SIZE           3102
#define IDS_PROP_VOLUME_NAME            3103
#define IDS_PROP_LOCAL_NAME             3200
#define IDS_PROP_PROVIDER               3201
#define IDS_OPTIONS                     4000
#define IDS_COMMENT                     4001
#define IDS_COMMENT2                    4002
#define IDS_SYSTEM                      4010

#define IDS_SPLITTING                   4020
#define IDS_SPLIT_CONFIRM_TITLE         4021
#define IDS_SPLIT_CONFIRM_MESSAGE       4022
#define IDS_SPLIT_VOL_MUST_BE_SMALLER   4023

#define IDS_COMBINE                     4030
#define IDS_COMBINE_TO                  4031
#define IDS_COMBINING                   4032
#define IDS_COMBINE_SELECT_ONE_FILE     4033
#define IDS_COMBINE_CANT_DETECT_SPLIT_FILE 4034
#define IDS_COMBINE_CANT_FIND_MORE_THAN_ONE_PART 4035

#define IDS_CHECKSUM_CALCULATING        4040
#define IDS_CHECKSUM_INFORMATION        4041
#define IDS_CHECKSUM_CRC_DATA           4042
#define IDS_CHECKSUM_CRC_DATA_NAMES     4043

#define IDS_SCANNING                    4050
#define IDS_PROPERTIES                  4060

#define IDS_MESSAGE_UNSUPPORTED_OPERATION_FOR_LONG_PATH_FOLDER  4301
#define IDS_SELECT_ONE_FILE             4302
#define IDS_SELECT_FILES                4303
#define IDS_FILE_EXIST                  4304
