#define IDS_UPDATE_NOT_SUPPORTED 199
#define IDS_CANNOT_CREATE_FOLDER  200
#define IDS_OPEN_IS_NOT_SUPORTED_ARCHIVE  201

#define IDS_MESSAGES_DIALOG_EXTRACT_MESSAGE_CRC  202
#define IDS_MESSAGES_DIALOG_EXTRACT_MESSAGE_DATA_ERROR  203
#define IDS_MESSAGES_DIALOG_EXTRACT_MESSAGE_UNSUPPORTED_METHOD  204
#define IDS_MESSAGES_DIALOG_EXTRACT_MESSAGE_CRC_ENCRYPTED  205
#define IDS_MESSAGES_DIALOG_EXTRACT_MESSAGE_DATA_ERROR_ENCRYPTED  206

#define IDS_EXTRACT_SET_FOLDER  207
#define IDS_MESSAGES_DIALOG_EXTRACT_MESSAGE_CANNOT_OPEN_FILE  208
#define IDS_PROGRESS_EXTRACTING  209

#define IDS_CANT_OPEN_ARCHIVE   103
#define IDS_CANT_OPEN_ENCRYPTED_ARCHIVE 104
#define IDS_MEM_ERROR 105
#define IDS_UNKNOWN_ERROR 106
#define IDS_UNSUPPORTED_ARCHIVE_TYPE 107

#define IDC_EXTRACT_RADIO_FULL_PATHNAMES 1041
#define IDC_EXTRACT_RADIO_CURRENT_PATHNAMES 1042
#define IDC_EXTRACT_RADIO_NO_PATHNAMES 1043

#define IDC_EXTRACT_RADIO_ASK_BEFORE_OVERWRITE 1051
#define IDC_EXTRACT_RADIO_OVERWRITE_WITHOUT_PROMPT 1052
#define IDC_EXTRACT_RADIO_SKIP_EXISTING_FILES 1053
#define IDC_EXTRACT_RADIO_AUTO_RENAME 1056
#define IDC_EXTRACT_RADIO_AUTO_RENAME_EXISTING 1057
