// Windows/System.h

#ifndef __WINDOWS_SYSTEM_H
#define __WINDOWS_SYSTEM_H

#include "../Common/Types.h"

namespace NWindows {
namespace NSystem {

UInt32 GetNumberOfProcessors();
UInt64 GetRamSize();

}}

#endif
