#define IDS_PROP_PATH                3
#define IDS_PROP_NAME                4
#define IDS_PROP_EXTENSION           5
#define IDS_PROP_IS_FOLDER           6
#define IDS_PROP_SIZE                7
#define IDS_PROP_PACKED_SIZE         8
#define IDS_PROP_ATTRIBUTES          9
#define IDS_PROP_CTIME              10
#define IDS_PROP_ATIME              11
#define IDS_PROP_MTIME              12
#define IDS_PROP_SOLID              13
#define IDS_PROP_C0MMENTED          14
#define IDS_PROP_ENCRYPTED          15
#define IDS_PROP_DICTIONARY_SIZE    16
#define IDS_PROP_SPLIT_BEFORE       17
#define IDS_PROP_SPLIT_AFTER        18
#define IDS_PROP_CRC                19
#define IDS_PROP_FILE_TYPE          20
#define IDS_PROP_ANTI               21
#define IDS_PROP_METHOD             22
#define IDS_PROP_HOST_OS            23
#define IDS_PROP_FILE_SYSTEM        24
#define IDS_PROP_USER               25
#define IDS_PROP_GROUP              26
#define IDS_PROP_BLOCK              27
#define IDS_PROP_COMMENT            28
#define IDS_PROP_POSITION           29
#define IDS_PROP_PREFIX             30
#define IDS_PROP_FOLDERS            31
#define IDS_PROP_FILES              32
#define IDS_PROP_VERSION            33
#define IDS_PROP_VOLUME             34
#define IDS_PROP_IS_VOLUME          35
#define IDS_PROP_OFFSET             36
#define IDS_PROP_LINKS              37
#define IDS_PROP_NUM_BLOCKS         38
#define IDS_PROP_NUM_VOLUMES        39

#define IDS_PROP_BIT64              41
#define IDS_PROP_BIG_ENDIAN         42
#define IDS_PROP_CPU                43
#define IDS_PROP_PHY_SIZE           44
#define IDS_PROP_HEADERS_SIZE       45
#define IDS_PROP_CHECKSUM           46
#define IDS_PROP_CHARACTS           47
#define IDS_PROP_VA                 48
#define IDS_PROP_ID                 49
#define IDS_PROP_SHORT_NAME         50
#define IDS_PROP_CREATOR_APP        51
#define IDS_PROP_SECTOR_SIZE        52
#define IDS_PROP_POSIX_ATTRIB       53
#define IDS_PROP_LINK               54
#define IDS_PROP_ERROR              55
