#include "SplitDialogRes.h"
#include "../../GuiCommon.rc"

#define xc 288
#define yc 96

IDD_DIALOG_SPLIT MY_RESIZE_DIALOG
CAPTION "Split File"
BEGIN
  LTEXT       "&Split to:", IDC_STATIC_SPLIT_PATH, m, m, xc, 8
  COMBOBOX    IDC_COMBO_SPLIT_PATH, m, 20, xc - bxsDots - m, 64, MY_COMBO_WITH_EDIT
  PUSHBUTTON  "...", IDC_BUTTON_SPLIT_PATH, xs - m - bxsDots, 18, bxsDots, bys, WS_GROUP
  LTEXT       "Split to &volumes,  bytes:", IDC_STATIC_SPLIT_VOLUME, m, 44, xc, 8
  COMBOBOX    IDC_COMBO_SPLIT_VOLUME, m, 56, 96, 52, MY_COMBO_WITH_EDIT
  OK_CANCEL
END
