#include "resource.h"

STRINGTABLE
BEGIN
  IDS_CONTEXT_EXTRACT     "Extract files..."
  IDS_CONTEXT_EXTRACT_HELP "Extracts files from the selected archive."
  IDS_CONTEXT_COMPRESS    "Add to archive..."
  IDS_CONTEXT_COMPRESS_HELP "Adds the selected items to archive."
  IDS_CONTEXT_OPEN        "Open archive"
  IDS_CONTEXT_OPEN_HELP   "Opens the selected archive."
  IDS_CONTEXT_TEST        "Test archive"
  IDS_CONTEXT_TEST_HELP   "Tests integrity of the selected archive."
  IDS_CONTEXT_CAPTION_HELP "7-Zip commands"
  IDS_CONTEXT_POPUP_CAPTION "7-Zip"
  IDS_CONTEXT_EXTRACT_HERE "Extract Here"
  IDS_CONTEXT_EXTRACT_HERE_HELP "Extracts files from the selected archive to current folder."
  IDS_CONTEXT_EXTRACT_TO  "Extract to {0}"
  IDS_CONTEXT_EXTRACT_TO_HELP "Extracts files to subfolder."
  IDS_CONTEXT_COMPRESS_TO "Add to {0}"
  IDS_CONTEXT_COMPRESS_TO_HELP "Adds the selected items to archive."
  IDS_CONTEXT_COMPRESS_EMAIL "Compress and email..."
  IDS_CONTEXT_COMPRESS_EMAIL_HELP "Compresses the selected items to archive and sends archive via email."
  IDS_CONTEXT_COMPRESS_TO_EMAIL "Compress to {0} and email"
  IDS_CONTEXT_COMPRESS_TO_EMAIL_HELP "Compresses the selected items to archive and sends archive via email."
  IDS_CONTEXT_FOLDER      "<Folder>"
  IDS_CONTEXT_ARCHIVE     "<Archive>"
END
