#include "PropertyNameRes.h"

LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US

STRINGTABLE
BEGIN
  IDS_PROP_PATH       "Path"
  IDS_PROP_NAME       "Name"
  IDS_PROP_EXTENSION  "Extension"
  IDS_PROP_IS_FOLDER  "Folder"
  IDS_PROP_SIZE       "Size"
  IDS_PROP_PACKED_SIZE "Packed Size"
  IDS_PROP_ATTRIBUTES "Attributes"
  IDS_PROP_CTIME "Created"
  IDS_PROP_ATIME "Accessed"
  IDS_PROP_MTIME "Modified"
  IDS_PROP_SOLID      "Solid"
  IDS_PROP_C0MMENTED  "Commented"
  IDS_PROP_ENCRYPTED  "Encrypted"
  IDS_PROP_DICTIONARY_SIZE "Dictionary Size"
  IDS_PROP_SPLIT_BEFORE "Split Before"
  IDS_PROP_SPLIT_AFTER "Split After"
  IDS_PROP_CRC        "CRC"
  IDS_PROP_FILE_TYPE  "Type"
  IDS_PROP_ANTI       "Anti"
  IDS_PROP_METHOD     "Method"
  IDS_PROP_HOST_OS    "Host OS"
  IDS_PROP_FILE_SYSTEM "File System"
  IDS_PROP_USER "User"
  IDS_PROP_GROUP "Group"
  IDS_PROP_BLOCK "Block"
  IDS_PROP_COMMENT "Comment"
  IDS_PROP_POSITION "Position"
  IDS_PROP_PREFIX "Path Prefix"
  IDS_PROP_FOLDERS "Folders"
  IDS_PROP_FILES "Files"
  IDS_PROP_VERSION "Version"
  IDS_PROP_VOLUME "Volume"
  IDS_PROP_IS_VOLUME "Multivolume"
  IDS_PROP_OFFSET "Offset"
  IDS_PROP_LINKS "Links"
  IDS_PROP_NUM_BLOCKS "Blocks"
  IDS_PROP_NUM_VOLUMES "Volumes"

  IDS_PROP_BIT64 "64-bit"
  IDS_PROP_BIG_ENDIAN "Big-endian"
  IDS_PROP_CPU "CPU"
  IDS_PROP_PHY_SIZE "Physical Size"
  IDS_PROP_HEADERS_SIZE "Headers Size"
  IDS_PROP_CHECKSUM "Checksum"
  IDS_PROP_CHARACTS "Characteristics"
  IDS_PROP_VA "Virtual Address"
  IDS_PROP_ID "ID"
  IDS_PROP_SHORT_NAME "Short Name"
  IDS_PROP_CREATOR_APP "Creator Application"
  IDS_PROP_SECTOR_SIZE "Sector Size"
  IDS_PROP_POSIX_ATTRIB "Mode"
  IDS_PROP_LINK "Link"
  IDS_PROP_ERROR "Error"
END
