#include "../../MyVersionInfo.rc"
#include <winnt.h>

#include "resource2.rc"
#include "../FileManager/resourceGui.rc"

MY_VERSION_INFO_APP("7-Zip GUI", "7zg")

IDI_ICON ICON "FM.ico"

#ifndef UNDER_CE
1 24 MOVEABLE PURE "7zG.exe.manifest"
#endif

#ifdef UNDER_CE
#include "../FileManager/PropertyName.rc"
#endif
#include "../FileManager/OverwriteDialog.rc"
#include "../FileManager/PasswordDialog.rc"
#include "../FileManager/ProgressDialog2.rc"
#include "Extract.rc"
