#include "PluginsPageRes.h"
#include "../../GuiCommon.rc"

#define xc 96
#define yc 80

IDD_PLUGINS  MY_PAGE
CAPTION "Plugins"
BEGIN
  LTEXT     "&Plugins:", IDC_PLUGINS_STATIC_PLUGINS, m, m, xc, 8
  CONTROL   "List1", IDC_PLUGINS_LIST, "SysListView32",
            LVS_REPORT | LVS_SHOWSELALWAYS | LVS_NOCOLUMNHEADER | LVS_NOSORTHEADER | WS_BORDER | WS_TABSTOP,
            m, 20, xc, 40
  PUSHBUTTON  "Options...", IDC_PLUGINS_BUTTON_OPTIONS, m, by, bxs, bys
END
