#include "CopyDialogRes.h"
#include "../../GuiCommon.rc"

#define xc 320
#define yc 144

#define y 40

IDD_DIALOG_COPY  MY_RESIZE_DIALOG
CAPTION "Copy"
{
  LTEXT       "",    IDC_COPY_STATIC,   m, m, xc, 8
  COMBOBOX           IDC_COPY_COMBO,    m, 20, xc - bxsDots - m, 65, MY_COMBO_WITH_EDIT
  PUSHBUTTON  "...", IDC_COPY_SET_PATH, xs - m - bxsDots, 18, bxsDots, bys, WS_GROUP
  LTEXT       "",    IDC_COPY_INFO,     m, y, xc, by - y - 1, SS_NOPREFIX | SS_LEFTNOWORDWRAP
  OK_CANCEL
}
