#include "MessagesDialogRes.h"
#include "../../GuiCommon.rc"

#define xc 440
#define yc 160

IDD_DIALOG_MESSAGES  MY_RESIZE_DIALOG
CAPTION "7-Zip: Diagnostic messages"
{
  DEFPUSHBUTTON "&Close", IDOK, bx, by, bxs, bys
  CONTROL "List1", IDC_MESSAGE_LIST, "SysListView32",
          LVS_REPORT | LVS_SHOWSELALWAYS | LVS_NOSORTHEADER | WS_BORDER | WS_TABSTOP,
          m, m, xc, yc - bys - m
}
