#define IDD_DIALOG_COMPRESS             551
#define IDD_DIALOG_COMPRESS_2           651
#define IDC_STATIC_COMPRESS_MEMORY      1022
#define IDC_STATIC_COMPRESS_MEMORY_DE   1023
#define IDC_STATIC_COMPRESS_MEMORY_VALUE 1027
#define IDC_STATIC_COMPRESS_MEMORY_DE_VALUE 1028
#define IDC_COMPRESS_COMBO_ARCHIVE      1072
#define IDC_COMPRESS_BUTTON_SET_ARCHIVE 1073
#define IDC_COMPRESS_COMBO_LEVEL        1074
#define IDC_COMPRESS_COMBO_UPDATE_MODE  1075
#define IDC_COMPRESS_COMBO_FORMAT       1076
#define IDC_COMPRESS_COMBO_VOLUME       1077
#define IDC_COMPRESS_COMBO_METHOD       1078
#define IDC_COMPRESS_COMBO_DICTIONARY   1079
#define IDC_COMPRESS_COMBO_ORDER        1080

#define IDC_COMPRESS_COMBO_SOLID        1081
#define IDC_COMPRESS_COMBO_THREADS      1082
#define IDC_COMPRESS_HARDWARE_THREADS   1083

#define IDC_COMPRESS_SFX                1090
#define IDC_COMPRESS_EDIT_PARAMETERS    1091

// #define IDC_COMPRESS_SOLID              1092
// #define IDC_COMPRESS_MULTI_THREAD       1093

#define IDC_STATIC_COMPRESS_ARCHIVE     1097
#define IDC_STATIC_COMPRESS_FORMAT      1098
#define IDC_STATIC_COMPRESS_LEVEL       1099
#define IDC_STATIC_COMPRESS_PARAMETERS  1100
#define IDC_STATIC_COMPRESS_UPDATE_MODE 1101
#define IDC_STATIC_COMPRESS_OPTIONS     1102
#define IDC_STATIC_COMPRESS_VOLUME      1103
#define IDC_STATIC_COMPRESS_METHOD      1104
#define IDC_STATIC_COMPRESS_DICTIONARY  1105
#define IDC_STATIC_COMPRESS_ORDER       1106

#define IDC_STATIC_COMPRESS_SOLID       1107
#define IDC_STATIC_COMPRESS_THREADS     1108

#define IDC_COMPRESS_ENCRYPTION         1110
#define IDC_STATIC_COMPRESS_PASSWORD1   1111
#define IDC_COMPRESS_EDIT_PASSWORD1     1112
#define IDC_STATIC_COMPRESS_PASSWORD2   1113
#define IDC_COMPRESS_EDIT_PASSWORD2     1114
#define IDC_COMPRESS_CHECK_SHOW_PASSWORD 1115

#define IDC_STATIC_COMPRESS_ENCRYPTION_METHOD 1120
#define IDC_COMPRESS_COMBO_ENCRYPTION_METHOD 1121

#define IDC_COMPRESS_CHECK_ENCRYPT_FILE_NAMES 1122

#define IDC_COMPRESS_SHARED    1130


#define IDS_OPEN_TYPE_ALL_FILES         80

#define IDS_METHOD_STORE                81
#define IDS_METHOD_NORMAL               82
#define IDS_METHOD_MAXIMUM              83
#define IDS_METHOD_FAST                 84
#define IDS_METHOD_FASTEST              85
#define IDS_METHOD_ULTRA                86

#define IDS_COMPRESS_NON_SOLID 88
#define IDS_COMPRESS_SOLID 89

#define IDS_COMPRESS_UPDATE_MODE_ADD    90
#define IDS_COMPRESS_UPDATE_MODE_UPDATE 91
#define IDS_COMPRESS_UPDATE_MODE_FRESH  92
#define IDS_COMPRESS_UPDATE_MODE_SYNCHRONIZE 93

#define IDS_COMPRESS_SPLIT_CONFIRM_MESSAGE   94

#define IDS_COMPRESS_SET_ARCHIVE_DIALOG_TITLE 96

#define IDS_PASSWORD_USE_ASCII               110
#define IDS_PASSWORD_PASSWORDS_DO_NOT_MATCH  111
#define IDS_PASSWORD_IS_TOO_LONG             112

#define IDS_COMPRESS_INCORRECT_VOLUME_SIZE   95
