#define IDD_SETTINGS                    543
#define IDD_SETTINGS_2                  643

#define IDC_SETTINGS_SHOW_DOTS          1000
#define IDC_SETTINGS_SHOW_REAL_FILE_ICONS 1001

#define IDC_SETTINGS_SHOW_SYSTEM_MENU   1010
#define IDC_SETTINGS_FULL_ROW           1011
#define IDC_SETTINGS_SHOW_GRID          1013
#define IDC_SETTINGS_SINGLE_CLICK       1014
#define IDC_SETTINGS_UNDERLINE          1015
#define IDC_SETTINGS_ALTERNATIVE_SELECTION 1016
#define IDC_SETTINGS_LARGE_PAGES 1017
