#include "PasswordDialogRes.h"
#include "../../GuiCommon.rc"

#define xc 140
#define yc 72

IDD_DIALOG_PASSWORD MY_DIALOG
CAPTION "Enter password"
BEGIN
  LTEXT    "&Enter password:", IDC_STATIC_PASSWORD_HEADER, m, m, xc, 8
  EDITTEXT  IDC_EDIT_PASSWORD, m, 20, xc, 14, ES_PASSWORD | ES_AUTOHSCROLL
  CONTROL  "&Show password", IDC_CHECK_PASSWORD_SHOW, MY_CHECKBOX, m, 42, xc, 10
  OK_CANCEL
END
