#define IDD_DIALOG_OVERWRITE             502
#define IDD_DIALOG_OVERWRITE_2           602

#define IDS_FILE_MODIFIED               600
#define IDS_FILE_SIZE                   601

#define IDC_STATIC_OVERWRITE_HEADER     1000

#define IDC_STATIC_OVERWRITE_QUESTION_BEGIN 1001
#define IDC_STATIC_OVERWRITE_QUESTION_END 1002

#define IDC_STATIC_OVERWRITE_OLD_FILE_ICON 1003
#define IDC_STATIC_OVERWRITE_NEW_FILE_ICON 1004

#define IDC_STATIC_OVERWRITE_OLD_FILE_SIZE_TIME 1005
#define IDC_STATIC_OVERWRITE_NEW_FILE_SIZE_TIME 1006

#define IDC_BUTTON_OVERWRITE_YES_TO_ALL 1010
#define IDC_BUTTON_OVERWRITE_NO_TO_ALL  1011
#define IDC_BUTTON_OVERWRITE_AUTO_RENAME 1012
